# AFT Output Elements.
# !!DO NOT EDIT!! This file was automatically generated by aft-compile v2.05a on Fri Jan  9 22:44:14 2004
# See http://www.maplefish.com/todd/aft.html for details.
package AFT_OUTPUT;

use vars qw ($filePreamble $filePostamble %elem %pragma_postvar);

$filePreamble = '';	# Holds preamble for output file.
$filePostamble = '';	# Holds postamble for output file.
%elem = ();		# Element commands for producing output file.%pragma_postvar = ();	# Variables for substitution post-filtering.


sub initElements {
	%pragma_postvar = (
	);
	%elem = (
		'StartFilteredVerbatim' =>	 '{\f1\fs20\verbatim',
		'^****' =>	 '',
		'Image-center' =>	 '',
		'StrikeLine' =>	 '%%--- %line%',
		'StartTeletype' =>	 '{\f1\fs20 ',
		'StartSmall' =>	 '{\fs16 ',
		'StartStrong' =>	 '{\b ',
		'TableHeader' =>	 '{\b %stuff%}\cell',
		'StartBulletList' =>	 '\bullist',
		'StartQuote' =>	 '\par\pard\li500\ri500 { ',
		'EndBulletListElement' =>	 '',
		'NumberedListElement' =>	 '\numitem',
		'EndEmphasis' =>	 '}',
		'URL' =>	 '{\cf1\ul %text%\cf0}',
		'EndNamedList' =>	 '\par}',
		'^**' =>	 '',
		'TableRowStart' =>	 '\intbl',
		'EndBulletList' =>	 '\endbullist',
		'****' =>	 '{\f0\fs20 {\par\pard\b %section%\par}}',
		'NBSPACE' =>	 '\~',
		'^*' =>	 '',
		'Target' =>	 '{\cf1\ul %text%\cf0}',
		'*' =>	 '{\f0\fs36 {\par\pard\b %section%\par}}',
		'EndTeletype' =>	 '}',
		'Image-right' =>	 '',
		'Title' =>	 '\par\pard \qc {\fs48\b %title%} \par\pard',
		'EndParagraph' =>	 '\par ',
		'interpolate' =>	 'no',
		'Author' =>	 '\par\pard \qc {\fs36\b %author%} \par\pard \par\pard',
		'TableRowEnd' =>	 '\row',
		'FullFilterFilteredVerbatim?' =>	 'Yes',
		'**' =>	 '{\f0\fs28 {\par\pard\b %section%\par}}',
		'EndNumberedListElement' =>	 '',
		'Image-left' =>	 '',
		'EndStrong' =>	 '}',
		'TOC' =>	 '',
		'EndBlockedVerbatim' =>	 '\endverbatim}',
		'Image' =>	 '',
		'NamedListElement' =>	 '\par\pard{\b %name%} \par\tx500\li500\fi-500 \tab',
		'StartNumberedList' =>	 '\numlist',
		'TableCaption' =>	 '\par\pard%caption%\par\par\pard\trowd\cellxx',
		'InternalReference' =>	 '{\cf1\ul %text%\cf0}',
		'StartNamedList' =>	 '{',
		'StartVerbatim' =>	 '{\f1\fs20\verbatim',
		'HorizontalLine' =>	 '{\emdash\emdash\emdash\emdash\emdash\emdash\emdash\emdash\emdash\emdash}',
		'EndNumberedList' =>	 '\endnumlist',
		'CommentLine' =>	 '',
		'StartBlockedVerbatim' =>	 '{\f1\fs20\verbatim',
		'EndTable' =>	 '\pard\par',
		'StartParagraph' =>	 '\par\pard ',
		'PreFilterVerbatim?' =>	 'Yes',
		'EndSect1' =>	 '',
		'Center' =>	 '\pard\qc {%center%} \par\pard ',
		'EndSect2' =>	 '',
		'EndFilteredVerbatim' =>	 '\endverbatim}',
		'EndSect3' =>	 '',
		'EndSect4' =>	 '',
		'EndNamedListElement' =>	 '',
		'PostProcessor' =>	 'exec "$^X @INC[0]/postrtf.pl $outputfile"',
		'EndVerbatim' =>	 '\endverbatim}',
		'StartEmphasis' =>	 '{\i ',
		'EndSmall' =>	 '}',
		'BulletListElement' =>	 '\bulitem',
		'StartTable' =>	 '\tabcols%columns%\endtabcols',
		'^***' =>	 '',
		'ID' =>	 'RTF',
		'EndQuote' =>	 '} \par\pard',
		'TableElement' =>	 '%stuff%\cell',
		'***' =>	 '{\f0\fs24 {\par\pard\b %section%\par}}',
	);

## Preamble:

$filePreamble = '{\rtf1\ansi\ansicpg1252\deff0\deftab720
{\fonttbl{\f0\froman Times New Roman;}{\f1\fmodern Courier;}{\f2\fswiss Arial;}}
{\info{\title %title%}{\author %author%}}
{\colortbl ;\red0\green0\blue255;}

\deflang1031

';

## Postamble:

$filePostamble = '}
';
}


## Prefilter subroutine:

sub prefilter {
   my ($line) = @_;
   $line=~s/\{\+(.*?)\+\}/%LeFtB%PlUs$1%PlUs%RiGhTB/g;
   $line=~s/\}\+(.*?)\+\{/%RiGhTB%PlUs$1%PlUs%LeFtB/g;
   $line=~s/\{\-(.*?)\-\}/%LeFtB%NeG$1%NeG%RiGhTB/g;
   $line=~s/\}\-(.*?)\-\{/%RiGhTB%NeG$1%NeG%LeFtB/g;
   $line=~s/\\/\\\\/g;
   $line=~s/\{/\\\{/g;
   $line=~s/\}/\\\}/g;
   $line=~s/%LeFtB/{/g;
   $line=~s/%RiGhTB/}/g;
   $line=~s/%PlUs/+/g;
   $line=~s/%NeG/-/g;
   return $line;
}

## Postfilter subroutine:

sub postfilter {
   my ($line) = @_;
   
   return $line;
}

1;
