# AFT Output Elements.
# !!DO NOT EDIT!! This file was automatically generated by aft-compile v2.05a on Fri Jan  9 22:44:13 2004
# See http://www.maplefish.com/todd/aft.html for details.
package AFT_OUTPUT;

use vars qw ($filePreamble $filePostamble %elem %pragma_postvar);

$filePreamble = '';	# Holds preamble for output file.
$filePostamble = '';	# Holds postamble for output file.
%elem = ();		# Element commands for producing output file.%pragma_postvar = ();	# Variables for substitution post-filtering.


sub initElements {
	%pragma_postvar = (
	);
	%elem = (
		'StartFilteredVerbatim' =>	 qq'<programlisting>',
		'EndNumberedListElement' =>	 qq'</listitem>',
		'EndStrong' =>	 qq'</emphasis>',
		'Image-left' =>	 qq'<inlinegraphic fileref="\%image\%" format="gif" align=left></inlinegraphic>',
		'EndBlockedVerbatim' =>	 qq']]></programlisting>	',
		'TOC' =>	 qq'',
		'Image' =>	 qq'<inlinegraphic fileref="\%image\%" format="gif"></inlinegraphic>',
		'Image-center' =>	 qq'<inlinegraphic fileref="\%image\%" format="gif" align=center></inlinegraphic>',
		'StrikeLine' =>	 qq'<comment> Striken: \%line\% </comment>',
		'StartTeletype' =>	 qq'<computeroutput>',
		'StartNumberedList' =>	 qq'<orderedlist>',
		'NamedListElement' =>	 qq'<varlistentry><term>\%name\%</term><listitem>',
		'StartSmall' =>	 qq'<small>',
		'TableCaption' =>	 qq'<title>\%caption\%</title>',
		'InternalReference' =>	 qq'<link linkend="\%target\%">\%text\%</link>',
		'StartVerbatim' =>	 qq'<literallayout>',
		'StartNamedList' =>	 qq'<variablelist>',
		'NoteRef' =>	 qq'<superscript>\%ref\%</superscript>',
		'StartStrong' =>	 qq'<emphasis role=bold>',
		'HorizontalLine' =>	 qq'',
		'TableHeader' =>	 qq'<entry>\%stuff\%</entry>',
		'EndNumberedList' =>	 qq'</orderedlist>',
		'StartBulletList' =>	 qq'<itemizedlist mark=opencircle>',
		'StartQuote' =>	 qq'<blockquote>',
		'EndBulletListElement' =>	 qq'</listitem>',
		'NumberedListElement' =>	 qq'<listitem>',
		'EndEmphasis' =>	 qq'</emphasis>',
		'Subscript' =>	 qq'<subscript>\%stuff\%</subscript>',
		'URL' =>	 qq'<ulink url="\%target\%">\%text\%</ulink>',
		'EndNamedList' =>	 qq'</variablelist>',
		'CommentLine' =>	 qq'<comment> \%line\% </comment>',
		'StartBlockedVerbatim' =>	 qq'<programlisting><![CDATA[',
		'TableRowStart' =>	 qq'<row>',
		'EndTable' =>	 qq'</table>',
		'StartParagraph' =>	 qq'<para>',
		'EndBulletList' =>	 qq'</itemizedlist>',
		'NBSPACE' =>	 qq'&nbsp',
		'PreFilterVerbatim?' =>	 qq'Yes',
		'Target' =>	 qq'<anchor id="\%target\%">\%text\%',
		'*' =>	 qq'<sec4 id="\%section\%"><title>\%section\%</title>',
		'EndSect1' =>	 qq'</sec1>',
		'EndTeletype' =>	 qq'</computeroutput>',
		'EndFilteredVerbatim' =>	 qq'</programlisting>',
		'EndSect2' =>	 qq'</sec2>',
		'Center' =>	 qq'\%center\%',
		'Image-right' =>	 qq'<inlinegraphic fileref="\%image\%" format="gif" align=right></inlinegraphic>',
		'EndSect3' =>	 qq'</sec3>',
		'EndNamedListElement' =>	 qq'</listitem></varlistentry>',
		'EndSect4' =>	 qq'</sec4>',
		'StartEmphasis' =>	 qq'<emphasis>',
		'EndVerbatim' =>	 qq'</literallayout>',
		'Title' =>	 qq'<title id="AFT-top">\%title\%</title>',
		'EndSmall' =>	 qq'</small>',
		'EndParagraph' =>	 qq'</para>',
		'BulletListElement' =>	 qq'<listitem>',
		'Superscript' =>	 qq'<superscript>\%stuff\%</supscript>',
		'ID' =>	 qq'DocBookArticle	',
		'Author' =>	 qq'<author>\%author\%</author>',
		'StartTable' =>	 qq'<table><tgroup cols="\%columns\%">',
		'EndQuote' =>	 qq'</blockquote>',
		'TableRowEnd' =>	 qq'</row>',
		'FullFilterFilteredVerbatim?' =>	 qq'Yes',
		'TableElement' =>	 qq'<entry>\%stuff\%</entry>',
	);

## Preamble:

$filePreamble = '<!doctype article PUBLIC "-//OASIS//DTD DocBook V3.1//EN">
<article>
<artheader>
	<author>%author%</author>
</artheader>
';

## Postamble:

$filePostamble = '</article>
';
}


## Prefilter subroutine:

sub prefilter {
   my ($line) = @_;
   $line=~s/\014//g;
   $line=~s/\&/\&amp;/g;
   $line=~s/\"/\&quot;/g;
   $line=~s/\>/\&gt;/g;
   $line=~s/\</\&lt;/g;
   $line=~s/\\\\/<br>/g;
   return $line;
}

## Postfilter subroutine:

sub postfilter {
   my ($line) = @_;
   
   return $line;
}

1;
