# AFT Output Elements.
# !!DO NOT EDIT!! This file was automatically generated by aft-compile v2.05a on Fri Jan  9 22:44:14 2004
# See http://www.maplefish.com/todd/aft.html for details.

# Using features from aft-html.dat.
package AFT_OUTPUT;

use vars qw ($filePreamble $filePostamble %elem %pragma_postvar);

$filePreamble = '';	# Holds preamble for output file.
$filePostamble = '';	# Holds postamble for output file.
%elem = ();		# Element commands for producing output file.%pragma_postvar = ();	# Variables for substitution post-filtering.


sub initElements {
	%pragma_postvar = (
		'stylesheet' =>	 'none',
	);
	%elem = (
		'StartFilteredVerbatim' =>	 qq'<pre>',
		'^****' =>	 qq'<br /><div id="\%section\%"><a href="#AFT-top">[Top]</a><h5>\%section\%</h5>',
		'Image-center' =>	 qq'<center><img src="\%image\%" alt="" align="bottom" /></center><br clear="all" />',
		'StrikeLine' =>	 qq'<!-- Striken: \%line\% -->',
		'StartTeletype' =>	 qq'<tt>',
		'StartSmall' =>	 qq'<small>',
		'StartStrong' =>	 qq'<strong>',
		'TableHeader' =>	 qq'<th>\%stuff\%</th>',
		'StartBulletList' =>	 qq'<ul>',
		'StartQuote' =>	 qq'<blockquote>',
		'EndBulletListElement' =>	 qq'</li>',
		'NumberedListElement' =>	 qq'<li>',
		'EndEmphasis' =>	 qq'</em>',
		'URL' =>	 qq'<a href="\%target\%">\%text\%</a>',
		'EndNamedList' =>	 qq'</dl>',
		'^**' =>	 qq'<br /><div id="\%section\%"><a href="#AFT-top">[Top]</a><h3><a name="\%section\%">\%section\%</a></h3>',
		'TableRowStart' =>	 qq'<tr>',
		'EndBulletList' =>	 qq'</ul>',
		'****' =>	 qq'<div id="\%section\%"><h5><a name="\%section\%">\%section\%</a></h5>',
		'NBSPACE' =>	 qq'&nbsp;',
		'^*' =>	 qq'<br /><div id="\%section\%"><a href="#AFT-top">[Top]</a><h2><a name="\%section\%">\%section\%</a></h2>',
		'Target' =>	 qq'<a name="\%target\%">\%text\%</a>',
		'*' =>	 qq'<div id="\%section\%"><h2><a name="\%section\%">\%section\%</a></h2>',
		'EndTeletype' =>	 qq'</tt>',
		'Image-right' =>	 qq'<img src="\%image\%" alt="" align="right" /><br clear="all" />',
		'BeginSectLevel1' =>	 qq'<!-- Start SectLevel1 -->',
		'Title' =>	 qq'<h1><a name="AFT-top"><center>\%title\%</center></a></h1>',
		'BeginSectLevel2' =>	 qq'<!-- Start SectLevel2 -->',
		'BeginSectLevel3' =>	 qq'<!-- Start SectLevel3 -->',
		'EndParagraph' =>	 qq'</p>',
		'BeginSectLevel4' =>	 qq'<!-- Start SectLevel4 -->',
		'Superscript' =>	 qq'<sup>\%stuff\%</sup>',
		'interpolate' =>	 qq'yes',
		'Author' =>	 qq'<br /><center><h2>\%author\%</h2></center>',
		'EndSectLevel1' =>	 qq'<!-- End SectLevel1 -->',
		'EndSectLevel2' =>	 qq'<!-- End SectLevel2 -->',
		'EndSectLevel3' =>	 qq'<!-- End SectLevel3 -->',
		'TableRowEnd' =>	 qq'</tr>',
		'EndSectLevel4' =>	 qq'<!-- End SectLevel4 -->',
		'FullFilterFilteredVerbatim?' =>	 qq'Yes',
		'**' =>	 qq'<div id="\%section\%"><h3><a name="\%section\%">\%section\%</a></h3>',
		'EndNumberedListElement' =>	 qq'</li>',
		'Image-left' =>	 qq'<img src="\%image\%" alt="" align="left" /><br clear="all" />',
		'EndStrong' =>	 qq'</strong>',
		'TOC' =>	 qq'',
		'EndBlockedVerbatim' =>	 qq'</pre>',
		'Image' =>	 qq'<img src="\%image\%" alt="" align="bottom" />',
		'NamedListElement' =>	 qq'<dt><strong>\%name\%</strong><dd>',
		'StartNumberedList' =>	 qq'<ol>',
		'TableCaption' =>	 qq'<caption>\%caption\%</caption>',
		'InternalReference' =>	 qq'<a href="#\%target\%">\%text\%</a>',
		'StartNamedList' =>	 qq'<dl>',
		'StartVerbatim' =>	 qq'<pre>',
		'NoteRef' =>	 qq'<sup>\%ref\%</sup>',
		'HorizontalLine' =>	 qq'<hr />',
		'EndNumberedList' =>	 qq'</ol>',
		'Subscript' =>	 qq'<sub>\%stuff\%</sub>',
		'CommentLine' =>	 qq'<!-- \%line\% -->',
		'StartBlockedVerbatim' =>	 qq'<pre>',
		'EndTable' =>	 qq'</table>',
		'StartParagraph' =>	 qq'<p class="Body">',
		'PreFilterVerbatim?' =>	 qq'Yes',
		'EndSect1' =>	 qq'</div><!--End Section h2-->',
		'Center' =>	 qq'<center>\%center\%</center>',
		'EndSect2' =>	 qq'</div><!--End Section h3-->',
		'EndFilteredVerbatim' =>	 qq'</pre>',
		'EndSect3' =>	 qq'</div><!--End Section h4-->',
		'EndSect4' =>	 qq'</div><!--End Section h5-->',
		'EndNamedListElement' =>	 qq'</dd></dt>',
		'EndVerbatim' =>	 qq'</pre>',
		'StartEmphasis' =>	 qq'<em>',
		'EndSmall' =>	 qq'</small>',
		'BulletListElement' =>	 qq'<li>',
		'ParagraphBreak' =>	 qq'<br />',
		'StartTable' =>	 qq'<!-- \%columns\% columns --><table>',
		'^***' =>	 qq'<br /><div id="\%section\%"><a href="#AFT-top">[Top]</a><h4><a name="\%section\%">\%section\%</a></h4>',
		'ID' =>	 qq'XHTML',
		'EndQuote' =>	 qq'</blockquote>',
		'TableElement' =>	 qq'<td>\%stuff\%</td>',
		'***' =>	 qq'<div id="\%section\%"><h4><a name="\%section\%">\%section\%</a></h4>',
	);

## Preamble:

$filePreamble = '<?xml version=\\'1.0\\' encoding=\\'iso-8859-1\\'?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
   "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" lang="en-US" xml:lang="en-US">
<head>
<title>%title%</title>
  <meta name="author" content="%author%" />
  <meta name="GENERATOR" content = "%version%" />
  <meta content="text/html; charset=iso-8859-1" />
  <meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1" />
</head>
<body>
';

## Postamble:

$filePostamble = '<br />
<hr />
<center><small><em>
%aft%
</em></small></center>
</body>
</html>
';
}


## Prefilter subroutine:

sub prefilter {
   my ($line) = @_;
   $line=~s/\&/\&amp;/g;
   $line=~s/\"/\&quot;/g;
   $line=~s/\>/\&gt;/g;
   $line=~s/\</\&lt;/g;
   $line=~s/\\\\/<br>/g;
   $line=~s/\\\\/<br/g;
   return $line;
}

## Postfilter subroutine:

sub postfilter {
   my ($line) = @_;
   $line=~s/\s{8}/\&nbsp;\&nbsp;/g;
   $line=~s/\t/\&nbsp;\&nbsp;/g;
   return $line;
}

1;
